<?php

declare(strict_types=1);

namespace Dedi\SyliusAvisVerifiesPlugin\Import;

use Dedi\SyliusAvisVerifiesPlugin\File\ValueObject\ImportFile;
use Dedi\SyliusAvisVerifiesPlugin\Repository\AvisVerifiesAwareRepositoryInterface;
use Doctrine\ORM\EntityManagerInterface;
use Psr\Log\LoggerInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Core\Model\ProductInterface;
use Sylius\Component\Core\Repository\CustomerRepositoryInterface;
use Sylius\Component\Core\Repository\OrderRepositoryInterface;
use Sylius\Component\Core\Repository\ProductReviewRepositoryInterface;
use Sylius\Component\Resource\Factory\FactoryInterface;
use Sylius\Component\Review\Model\ReviewerInterface;
use Sylius\Component\Review\Model\ReviewInterface;

final class ReviewFileImporter implements ReviewFileImporterInterface
{
    private ProductReviewRepositoryInterface $productReviewRepository;

    private FactoryInterface $productReviewFactory;

    private AvisVerifiesAwareRepositoryInterface $avisVerifiesAwareRepository;

    private OrderRepositoryInterface $orderRepository;

    private CustomerRepositoryInterface $customerRepository;

    private EntityManagerInterface $entityManager;

    private LoggerInterface $logger;

    public function __construct(
        ProductReviewRepositoryInterface $productReviewRepository,
        FactoryInterface $productReviewFactory,
        AvisVerifiesAwareRepositoryInterface $avisVerifiesAwareRepository,
        OrderRepositoryInterface $orderRepository,
        CustomerRepositoryInterface $customerRepository,
        EntityManagerInterface $entityManager,
        LoggerInterface $logger
    ) {
        $this->productReviewRepository = $productReviewRepository;
        $this->productReviewFactory = $productReviewFactory;
        $this->avisVerifiesAwareRepository = $avisVerifiesAwareRepository;
        $this->customerRepository = $customerRepository;
        $this->orderRepository = $orderRepository;
        $this->entityManager = $entityManager;
        $this->logger = $logger;
    }

    public function import(ImportFile $importFile): void
    {
        $xml = \simplexml_load_file($importFile->getLocalFile()->getPathname());

        $i = 0;

        foreach ($xml->review as $reviewNode) {
            $productId = (int) $reviewNode->product_ref;
            $orderNumber = (string) $reviewNode->order_ref;
            $email = isset($reviewNode->email) ? (string) $reviewNode->email : '';

            $product = $this->avisVerifiesAwareRepository->findOneByAvisVerifiesId($productId);

            if (null === $product) {
                $this->logger->error(sprintf('Unable to find product with avisVerifiesId %d', $productId));

                continue;
            }

            $customer = $this->getReviewer($orderNumber, $email);

            if (null === $customer) {
                $this->logger->error(sprintf(
                    'Unable to find customer for order number %d and email %s',
                    $orderNumber,
                    $email
                ));

                continue;
            }

            $this->logger->info(sprintf(
                'Create/update review for customer id %d and product id %d',
                $customer->getId(),
                $product->getId()
            ));

            $review = $this->findOrCreateReview($product, $customer);
            $date = new \DateTime((string) $reviewNode->review_date);
            $review->setComment((string) $reviewNode->review);
            $review->setRating((int) $reviewNode->rate);
            $review->setCreatedAt($date);
            $review->setUpdatedAt($date);

            $attributes = $reviewNode->attributes();
            switch ((string) $attributes['action']) {
                case 'NEW':
                    $review->setStatus(ReviewInterface::STATUS_ACCEPTED);

                    break;
                case 'DELETE':
                    $review->setStatus(ReviewInterface::STATUS_REJECTED);

                    break;
            }

            $this->entityManager->persist($review);

            ++$i;
            if ($i % 20 === 0) {
                $this->entityManager->flush();
                $i = 0;
            }
        }

        $this->entityManager->flush();
    }

    private function findOrCreateReview(ProductInterface $product, ReviewerInterface $customer): ReviewInterface
    {
        /** @var ReviewInterface|null $review */
        $review = $this->productReviewRepository->findOneBy([
            'reviewSubject' => $product->getId(),
            'author' => $customer->getId(),
        ]);

        if (null === $review) {
            /** @var ReviewInterface $review */
            $review = $this->productReviewFactory->createNew();
            $review->setReviewSubject($product);
            $review->setAuthor($customer);
        }

        return $review;
    }

    private function getReviewer(string $orderNumber, string $email = ''): ?ReviewerInterface
    {
        /** @var OrderInterface|null $order */
        $order = $this->orderRepository->findOneByNumber($orderNumber);

        if (null !== $order) {
            /** @var ReviewerInterface $reviewer */
            $reviewer = $order->getCustomer();

            return $reviewer;
        }

        if ('' === $email) {
            return null;
        }

        return $this->customerRepository->findOneBy(['email' => $email]);
    }
}
